// Package: lime.math.Vector4


// Class: lime.math.Vector4

var Vector4 = (function () {
var Vector4 = function(x,y,z,w) {
	if(w == null) {
		w = 0.;
	}
	if(z == null) {
		z = 0.;
	}
	if(y == null) {
		y = 0.;
	}
	if(x == null) {
		x = 0.;
	}
	this.w = w;
	this.x = x;
	this.y = y;
	this.z = z;
};
Vector4.prototype = {
    "__properties__": {"get_length":"get_length","get_lengthSquared":"get_lengthSquared"},
  copyFrom: function(sourceVector4) {
		this.x = sourceVector4.x;
		this.y = sourceVector4.y;
		this.z = sourceVector4.z;
	},
  normalize: function() {
		var l = Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
		if(l != 0) {
			this.x /= l;
			this.y /= l;
			this.z /= l;
		}
		return l;
	},
  z: null,
  subtract: function(a) {
		return new Vector4(this.x - a.x,this.y - a.y,this.z - a.z);
	},
  get_lengthSquared: function() {
		return this.x * this.x + this.y * this.y + this.z * this.z;
	},
  w: null,
  setTo: function(xa,ya,za) {
		this.x = xa;
		this.y = ya;
		this.z = za;
	},
  lengthSquared: null,
  crossProduct: function(a) {
		return new Vector4(this.y * a.z - this.z * a.y,this.z * a.x - this.x * a.z,this.x * a.y - this.y * a.x,1);
	},
  negate: function() {
		this.x *= -1;
		this.y *= -1;
		this.z *= -1;
	},
  add: function(a) {
		return new Vector4(this.x + a.x,this.y + a.y,this.z + a.z);
	},
  incrementBy: function(a) {
		this.x += a.x;
		this.y += a.y;
		this.z += a.z;
	},
  get_length: function() {
		return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
	},
  project: function() {
		this.x /= this.w;
		this.y /= this.w;
		this.z /= this.w;
	},
  scaleBy: function(s) {
		this.x *= s;
		this.y *= s;
		this.z *= s;
	},
  length: null,
  clone: function() {
		return new Vector4(this.x,this.y,this.z,this.w);
	},
  toString: function() {
		return "Vector4(" + this.x + ", " + this.y + ", " + this.z + ")";
	},
  decrementBy: function(a) {
		this.x -= a.x;
		this.y -= a.y;
		this.z -= a.z;
	},
  y: null,
  dotProduct: function(a) {
		return this.x * a.x + this.y * a.y + this.z * a.z;
	},
  equals: function(toCompare,allFour) {
		if(allFour == null) {
			allFour = false;
		}
		if(this.x == toCompare.x && this.y == toCompare.y && this.z == toCompare.z) {
			if(!(!allFour)) {
				return this.w == toCompare.w;
			} else {
				return true;
			}
		} else {
			return false;
		}
	},
  x: null,
  nearEquals: function(toCompare,tolerance,allFour) {
		if(allFour == null) {
			allFour = false;
		}
		if(Math.abs(this.x - toCompare.x) < tolerance && Math.abs(this.y - toCompare.y) < tolerance && Math.abs(this.z - toCompare.z) < tolerance) {
			if(!(!allFour)) {
				return Math.abs(this.w - toCompare.w) < tolerance;
			} else {
				return true;
			}
		} else {
			return false;
		}
	},
 __class__: Vector4
};
Vector4.__name__ = "lime.math.Vector4";
Vector4.X_AXIS = null;
Vector4.get_X_AXIS = function() {
	return new Vector4(1,0,0);
};
Vector4.Z_AXIS = null;
Vector4.angleBetween = function(a,b) {
	var a0 = new Vector4(a.x,a.y,a.z,a.w);
	var l = Math.sqrt(a0.x * a0.x + a0.y * a0.y + a0.z * a0.z);
	if(l != 0) {
		a0.x /= l;
		a0.y /= l;
		a0.z /= l;
	}
	var b0 = new Vector4(b.x,b.y,b.z,b.w);
	var l1 = Math.sqrt(b0.x * b0.x + b0.y * b0.y + b0.z * b0.z);
	if(l1 != 0) {
		b0.x /= l1;
		b0.y /= l1;
		b0.z /= l1;
	}
	return Math.acos(a0.x * b0.x + a0.y * b0.y + a0.z * b0.z);
};
Vector4.get_Z_AXIS = function() {
	return new Vector4(0,0,1);
};
Vector4.Y_AXIS = null;
Vector4.get_Y_AXIS = function() {
	return new Vector4(0,1,0);
};
Vector4.distance = function(pt1,pt2) {
	var x = pt2.x - pt1.x;
	var y = pt2.y - pt1.y;
	var z = pt2.z - pt1.z;
	return Math.sqrt(x * x + y * y + z * z);
};
Object.defineProperties(Vector4.prototype, {
	"lengthSquared": { get: Vector4.prototype.get_lengthSquared },
	"length": { get: Vector4.prototype.get_length },
});



return Vector4;
}());
exports.default = exports.Vector4 = Vector4;
