// Package: lime.text.Font
var lime_utils_Assets = require("../../lime/utils/Assets").Assets;
var $bind = require("../../bind_stub").bind_stub;
var lime_app_Promise = require("../../lime/app/Promise").Promise;
var lime_app_Future = require("../../lime/app/Future").Future;
var lime_net__HTTPRequest_lime_text_Font = require("../../lime/net/_HTTPRequest_lime_text_Font")._HTTPRequest_lime_text_Font;
var lime_utils_Log = require("../../lime/utils/Log").Log;


// Class: lime.text.Font

var Font = (function () {
var Font = function(name) {
	if(name != null) {
		this.name = name;
	}
	if(this.__fontID != null) {
		if(lime_utils_Assets.isLocal(this.__fontID)) {
			this.__fromBytes(lime_utils_Assets.getBytes(this.__fontID));
		}
	} else if(this.__fontPath != null) {
		this.__fromFile(this.__fontPath);
	}
};
Font.prototype = {
    "__properties__": {"get_ascender":"get_ascender","get_descender":"get_descender","get_height":"get_height","get_numGlyphs":"get_numGlyphs","get_underlinePosition":"get_underlinePosition","get_underlineThickness":"get_underlineThickness","get_unitsPerEM":"get_unitsPerEM"},
  underlinePosition: null,
  __fontID: null,
  __fontPath: null,
  __loadFromName: function(name) {
		var _gthis = this;
		var promise = new lime_app_Promise();
		this.name = name;
		var ua = window.navigator.userAgent.toLowerCase();
		var isSafari = ua.indexOf(" safari/") >= 0 && ua.indexOf(" chrome/") < 0;
		if(!isSafari && (window.document.fonts && ($_=window.document.fonts,$bind($_,$_.load)))) {
			window.document.fonts.load("1em '" + name + "'").then(function(_) {
				promise.complete(_gthis);
			},function(_1) {
				lime_utils_Log.warn("Could not load web font \"" + name + "\"",{ fileName : "Font.hx", lineNumber : 482, className : "lime.text.Font", methodName : "__loadFromName"});
				promise.complete(_gthis);
			});
		} else {
			var node1 = Font.__measureFontNode("'" + name + "', sans-serif");
			var node2 = Font.__measureFontNode("'" + name + "', serif");
			var width1 = node1.offsetWidth;
			var width2 = node2.offsetWidth;
			var interval = -1;
			var timeout = 3000;
			var intervalLength = 50;
			var intervalCount = 0;
			var loaded;
			var timeExpired;
			var checkFont = function() {
				intervalCount += 1;
				if(node1.offsetWidth == width1) {
					loaded = node2.offsetWidth != width2;
				} else {
					loaded = true;
				}
				timeExpired = intervalCount * intervalLength >= timeout;
				if(loaded || timeExpired) {
					window.clearInterval(interval);
					node1.parentNode.removeChild(node1);
					node2.parentNode.removeChild(node2);
					node1 = null;
					node2 = null;
					if(timeExpired) {
						lime_utils_Log.warn("Could not load web font \"" + name + "\"",{ fileName : "Font.hx", lineNumber : 518, className : "lime.text.Font", methodName : "__loadFromName"});
					}
					promise.complete(_gthis);
				}
			};
			interval = window.setInterval(checkFont,intervalLength);
		}
		return promise.future;
	},
  get_underlinePosition: function() {
		return 0;
	},
  numGlyphs: null,
  renderGlyph: function(glyph,fontSize) {
		return null;
	},
  get_ascender: function() {
		return 0;
	},
  __fromBytes: function(bytes) {
		this.__fontPath = null;
	},
  get_descender: function() {
		return 0;
	},
  unitsPerEM: null,
  __setSize: function(size) {
	},
  height: null,
  ascender: null,
  name: null,
  __fromFile: function(path) {
		this.__fontPath = path;
	},
  getGlyph: function(character) {
		return -1;
	},
  underlineThickness: null,
  src: null,
  get_underlineThickness: function() {
		return 0;
	},
  get_unitsPerEM: function() {
		return 0;
	},
  getGlyphs: function(characters) {
		if(characters == null) {
			characters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^`'\"/\\&*()[]{}<>|:;_-+=?,. ";
		}
		return null;
	},
  decompose: function() {
		return null;
	},
  getGlyphMetrics: function(glyph) {
		return null;
	},
  descender: null,
  get_height: function() {
		return 0;
	},
  get_numGlyphs: function() {
		return 0;
	},
  renderGlyphs: function(glyphs,fontSize) {
		return null;
	},
 __class__: Font
};
Font.__name__ = "lime.text.Font";
Font.__measureFontNode = function(fontFamily) {
	var node = window.document.createElement("span");
	node.setAttribute("aria-hidden","true");
	var text = window.document.createTextNode("BESbswy");
	node.appendChild(text);
	var style = node.style;
	style.display = "block";
	style.position = "absolute";
	style.top = "-9999px";
	style.left = "-9999px";
	style.fontSize = "300px";
	style.width = "auto";
	style.height = "auto";
	style.lineHeight = "normal";
	style.margin = "0";
	style.padding = "0";
	style.fontVariant = "normal";
	style.whiteSpace = "nowrap";
	style.fontFamily = fontFamily;
	window.document.body.appendChild(node);
	return node;
};
Font.loadFromFile = function(path) {
	var request = new lime_net__HTTPRequest_lime_text_Font();
	return request.load(path).then(function(font) {
		if(font != null) {
			return lime_app_Future.withValue(font);
		} else {
			return lime_app_Future.withError("");
		}
	});
};
Font.loadFromName = function(path) {
	var font = new Font();
	return font.__loadFromName(path);
};
Font.fromFile = function(path) {
	if(path == null) {
		return null;
	}
	var font = new Font();
	font.__fromFile(path);
	return font;
};
Font.loadFromBytes = function(bytes) {
	return lime_app_Future.withValue(Font.fromBytes(bytes));
};
Font.fromBytes = function(bytes) {
	if(bytes == null) {
		return null;
	}
	var font = new Font();
	font.__fromBytes(bytes);
	return font;
};
Object.defineProperties(Font.prototype, {
	"ascender": { get: Font.prototype.get_ascender },
	"underlinePosition": { get: Font.prototype.get_underlinePosition },
	"underlineThickness": { get: Font.prototype.get_underlineThickness },
	"numGlyphs": { get: Font.prototype.get_numGlyphs },
	"descender": { get: Font.prototype.get_descender },
	"unitsPerEM": { get: Font.prototype.get_unitsPerEM },
	"height": { get: Font.prototype.get_height },
});



return Font;
}());
exports.default = exports.Font = Font;
