// Package: lime.utils.AssetManifest
var HxOverrides = require("../../HxOverrides").HxOverrides;
var lime_utils__Bytes_Bytes_Impl_ = require("../../lime/utils/_Bytes/Bytes_Impl_").Bytes_Impl_;
var StringTools = require("../../StringTools").StringTools;
var haxe_Serializer = require("../../haxe/Serializer").Serializer;
var lime_app_Future = require("../../lime/app/Future").Future;
var haxe_io_Path = require("../../haxe/io/Path").Path;
var haxe_Unserializer = require("../../haxe/Unserializer").Unserializer;


// Class: lime.utils.AssetManifest

var AssetManifest = (function () {
var AssetManifest = function() {
	this.assets = [];
	this.libraryArgs = [];
	this.version = 2;
};
AssetManifest.prototype = {
  name: null,
  serialize: function() {
		var manifestData = { };
		manifestData.version = this.version;
		manifestData.libraryType = this.libraryType;
		manifestData.libraryArgs = this.libraryArgs;
		manifestData.name = this.name;
		manifestData.assets = haxe_Serializer.run(this.assets);
		return JSON.stringify(manifestData);
	},
  assets: null,
  libraryArgs: null,
  libraryType: null,
  version: null,
  rootPath: null,
 __class__: AssetManifest
};
AssetManifest.__name__ = "lime.utils.AssetManifest";
AssetManifest.__resolveRootPath = function(rootPath,path) {
	if(rootPath != null) {
		return rootPath;
	}
	var queryIndex = path.indexOf("?");
	if(queryIndex > -1) {
		rootPath = HxOverrides.substr(path,0,queryIndex);
	} else {
		rootPath = path;
	}
	StringTools.replace(rootPath,"\\","/");
	while(StringTools.endsWith(rootPath,"/")) {
		if(rootPath == "/") {
			return rootPath;
		}
		rootPath = HxOverrides.substr(rootPath,0,rootPath.length - 1);
	}
	if(StringTools.endsWith(rootPath,".bundle")) {
		return rootPath;
	} else {
		return haxe_io_Path.directory(rootPath);
	}
};
AssetManifest.__resolvePath = function(path) {
	if(path == null) {
		return null;
	}
	var queryIndex = path.indexOf("?");
	var basePath;
	if(queryIndex > -1) {
		basePath = HxOverrides.substr(path,0,queryIndex);
	} else {
		basePath = path;
	}
	StringTools.replace(basePath,"\\","/");
	while(StringTools.endsWith(basePath,"/")) basePath = HxOverrides.substr(basePath,0,basePath.length - 1);
	if(StringTools.endsWith(basePath,".bundle")) {
		if(queryIndex > -1) {
			return basePath + "/library.json" + HxOverrides.substr(path,queryIndex,null);
		} else {
			return basePath + "/library.json";
		}
	} else {
		return path;
	}
};
AssetManifest.parse = function(data,rootPath) {
	if(data == null || data == "") {
		return null;
	}
	var manifestData = JSON.parse(data);
	var manifest = new AssetManifest();
	manifest.name = manifestData.name;
	manifest.libraryType = manifestData.libraryType;
	manifest.libraryArgs = manifestData.libraryArgs;
	manifest.assets = haxe_Unserializer.run(manifestData.assets);
	if(rootPath != null) {
		manifest.rootPath = rootPath;
	}
	return manifest;
};
AssetManifest.loadFromBytes = function(bytes,rootPath) {
	return lime_app_Future.withValue(AssetManifest.fromBytes(bytes,rootPath));
};
AssetManifest.loadFromFile = function(path,rootPath) {
	path = AssetManifest.__resolvePath(path);
	rootPath = AssetManifest.__resolveRootPath(rootPath,path);
	if(path == null) {
		return null;
	}
	return lime_utils__Bytes_Bytes_Impl_.loadFromFile(path).then(function(bytes) {
		return lime_app_Future.withValue(AssetManifest.fromBytes(bytes,rootPath));
	});
};
AssetManifest.fromBytes = function(bytes,rootPath) {
	if(bytes != null) {
		return AssetManifest.parse(bytes.getString(0,bytes.length),rootPath);
	} else {
		return null;
	}
};
AssetManifest.fromFile = function(path,rootPath) {
	path = AssetManifest.__resolvePath(path);
	rootPath = AssetManifest.__resolveRootPath(rootPath,path);
	if(path == null) {
		return null;
	}
	return AssetManifest.fromBytes(lime_utils__Bytes_Bytes_Impl_.fromFile(path),rootPath);
};


return AssetManifest;
}());
exports.default = exports.AssetManifest = AssetManifest;
