// Package: openfl._internal.stage3D.atf.ATFReader
var haxe_io_Bytes = require("../../../../haxe/io/Bytes").Bytes;
var js__Boot_HaxeError = require("../../../../js/_Boot/HaxeError").HaxeError;
var openfl_utils__ByteArray_ByteArray_Impl_ = require("../../../../openfl/utils/_ByteArray/ByteArray_Impl_").ByteArray_Impl_;
var js_Boot = require("../../../../js/Boot").Boot;
var _UInt_UInt_Impl_ = require("../../../../_UInt/UInt_Impl_").UInt_Impl_;
var openfl_errors_IllegalOperationError = require("../../../../openfl/errors/IllegalOperationError").IllegalOperationError;
var lime_utils__DataPointer_DataPointer_Impl_ = require("../../../../lime/utils/_DataPointer/DataPointer_Impl_").DataPointer_Impl_;


// Class: openfl._internal.stage3D.atf.ATFReader

var ATFReader = (function () {
var ATFReader = function(data,byteArrayOffset) {
	this.version = 0;
	data.position = byteArrayOffset;
	var signature = data.readUTFBytes(3);
	data.position = byteArrayOffset;
	if(signature != "ATF") {
		throw new js__Boot_HaxeError(new openfl_errors_IllegalOperationError("ATF signature not found"));
	}
	var length = 0;
	if(data.b[byteArrayOffset + 6] == 255) {
		this.version = data.b[byteArrayOffset + 7];
		data.position = byteArrayOffset + 8;
		length = this.__readUInt32(data);
	} else {
		this.version = 0;
		data.position = byteArrayOffset + 3;
		length = this.__readUInt24(data);
	}
	if(js_Boot.__cast(byteArrayOffset + length , Int) > openfl_utils__ByteArray_ByteArray_Impl_.get_length(data)) {
		throw new js__Boot_HaxeError(new openfl_errors_IllegalOperationError("ATF length exceeds byte array length"));
	}
	this.data = data;
};
ATFReader.prototype = {
  readTextures: function(uploadCallback) {
		var gpuFormats = this.version < 3 ? 3 : 4;
		var sideCount = this.cubeMap ? 6 : 1;
		var _g1 = 0;
		var _g = sideCount;
		while(_g1 < _g) {
			var side = _g1++;
			var _g3 = 0;
			var _g2 = this.mipCount;
			while(_g3 < _g2) {
				var level = _g3++;
				var _g5 = 0;
				var _g4 = gpuFormats;
				while(_g5 < _g4) {
					var gpuFormat = _g5++;
					var blockLength = this.version == 0 ? this.__readUInt24(this.data) : this.__readUInt32(this.data);
					if(_UInt_UInt_Impl_.gt(this.data.position + blockLength,openfl_utils__ByteArray_ByteArray_Impl_.get_length(this.data))) {
						throw new js__Boot_HaxeError(new openfl_errors_IllegalOperationError("Block length exceeds ATF file length"));
					}
					if(_UInt_UInt_Impl_.gt(blockLength,0)) {
						var bytes = new haxe_io_Bytes(new ArrayBuffer(blockLength));
						this.data.readBytes(openfl_utils__ByteArray_ByteArray_Impl_.fromBytes(bytes),0,blockLength);
						uploadCallback(side,level,gpuFormat,this.width >> level,this.height >> level,blockLength,lime_utils__DataPointer_DataPointer_Impl_.fromBytes(bytes));
					}
				}
			}
		}
	},
  __readUInt32: function(data) {
		var value = data.readUnsignedByte() << 24;
		value = value | data.readUnsignedByte() << 16;
		value = value | data.readUnsignedByte() << 8;
		value = value | data.readUnsignedByte();
		return value;
	},
  cubeMap: null,
  __readUInt24: function(data) {
		var value = data.readUnsignedByte() << 16;
		value = value | data.readUnsignedByte() << 8;
		value = value | data.readUnsignedByte();
		return value;
	},
  mipCount: null,
  readHeader: function(__width,__height,cubeMap) {
		var tdata = this.data.readUnsignedByte();
		var type = tdata >> 7;
		if(!cubeMap && type != 0) {
			throw new js__Boot_HaxeError(new openfl_errors_IllegalOperationError("ATF Cube map not expected"));
		}
		if(cubeMap && type != 1) {
			throw new js__Boot_HaxeError(new openfl_errors_IllegalOperationError("ATF Cube map expected"));
		}
		this.cubeMap = cubeMap;
		var atfFormat = tdata & 127;
		if(atfFormat != 3 && atfFormat != 5) {
			throw new js__Boot_HaxeError(new openfl_errors_IllegalOperationError("Only ATF block compressed textures without JPEG-XR+LZMA are supported"));
		}
		this.width = 1 << this.data.readUnsignedByte();
		this.height = 1 << this.data.readUnsignedByte();
		if(this.width != __width || this.height != __height) {
			throw new js__Boot_HaxeError(new openfl_errors_IllegalOperationError("ATF width and height dont match"));
		}
		this.mipCount = this.data.readUnsignedByte();
		return atfFormat == 3;
	},
  data: null,
  version: null,
  width: null,
  height: null,
 __class__: ATFReader
};
ATFReader.__name__ = "openfl._internal.stage3D.atf.ATFReader";


return ATFReader;
}());
exports.default = exports.ATFReader = ATFReader;
